<?php

namespace Drupal\push_framework;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\user\UserInterface;

/**
 * Interface for push_framework plugins.
 */
interface ChannelPluginInterface extends PluginInspectionInterface {

  public const RESULT_STATUS_SUCCESS = 'success';
  public const RESULT_STATUS_RETRY = 'retry';
  public const RESULT_STATUS_FAILED = 'failed';

  /**
   * Returns the translated plugin label.
   *
   * @return string
   *   The translated title.
   */
  public function label(): string;

  /**
   * Initialize the plugin.
   */
  public function init(): void;

  /**
   * @param \Drupal\user\UserInterface $user
   *
   * @return bool
   */
  public function applicable(UserInterface $user): bool;

  /**
   * Sends the given notification.
   *
   * @param \Drupal\user\UserInterface $user
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   * @param int $attempt
   *
   * @return string
   *   One fo the RESULT_STATUS_ constants above.
   */
  public function send(UserInterface $user, ContentEntityInterface $entity, $attempt): string;

}
