<?php

namespace Drupal\push_framework\Plugin\SaneRecipientSelection;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\push_framework\ChannelPluginManager;
use Drupal\sane\PayloadInterface;
use Drupal\sane\RecipientSelectionBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the sane.
 */
abstract class DirectPush extends RecipientSelectionBase {

  /**
   * @var \Drupal\push_framework\ChannelPluginManager
   */
  protected $channelPluginManager;

  /**
   * PluginBase constructor.
   *
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param \Drupal\push_framework\ChannelPluginManager $channel_plugin_manager
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, ConfigFactoryInterface $config_factory, ChannelPluginManager $channel_plugin_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $config_factory);
    $this->channelPluginManager = $channel_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('push_framework.channel.plugin.manager')
    );
  }

  /**
   * @return string
   */
  abstract protected function directPushChannelId(): string;

  /**
   * {@inheritdoc}
   */
  public function getRecipients(PayloadInterface $payload): array {
    $this->push($payload);
    return [];
  }

  /**
   * @param \Drupal\sane\PayloadInterface $payload
   */
  protected function push(PayloadInterface $payload): void {
    try {
      /** @var \Drupal\push_framework\ChannelPluginInterface $channel */
      $channel = $this->channelPluginManager->createInstance($this->directPushChannelId());
      /** @var \Drupal\user\UserInterface $user */
      $user = $this->entityTypeManager->getStorage('user')->load(1);
    }
    catch (PluginException $e) {
      // TODO: Log this exception.
      return;
    }
    $channel->send($user, $payload->getEntity(), 0);
  }

}
